@@ -54,6 +54,10 @@
 #define EVIOCGRAB _IOW('E', 0x90, int)
 #endif
 
+#ifndef BTN_TASK
+#define BTN_TASK 0x117
+#endif
+
 #ifndef EV_SYN
 #define EV_SYN EV_RST
 #endif
@@ -160,6 +164,7 @@ EvdevReadInput(InputInfoPtr pInfo)
             case BTN_EXTRA:
             case BTN_FORWARD:
             case BTN_BACK:
+            case BTN_TASK:
                 xf86PostButtonEvent(pInfo->dev, 0, ev.code - BTN_LEFT + 5,
                                     value, 0, 0);
                 break;
