@@ -90,11 +90,11 @@
  *  - Load "speedo" module.
  *  - Ready to DRI.
  *  - Load xtt module instead of freetype module.
- *  - Add font path "/fonts/TrueType/" and "/fonts/freefont/".
+ *  - Add font path "/TrueType/" and "/freefont/".
  *  Chisato Yamauchi(cyamauch@phyas.aichi-edu.ac.jp)
  */
 /* $XConsortium: xf86config.c /main/21 1996/10/28 05:43:57 kaleb $ */
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/xf86config/xorgconfig.c,v 1.6.2.2 2005/01/12 20:09:51 gisburn Exp $ */
+/* $XdotOrg: xc/programs/Xserver/hw/xfree86/xf86config/xorgconfig.c,v 1.14 2005/05/28 00:08:03 alanc Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -127,14 +127,6 @@ static int getuid() { return 0; }
  */
 #define XFREE86_VERSION 400
 
-/*
- * This is the filename of the temporary XF86Config file that is written
- * when the program is told to probe clocks (which can only happen for
- * root).
- */
-#define TEMPORARY_XF86CONFIG_DIR_PREFIX "/tmp/.xf86config"
-#define TEMPORARY_XF86CONFIG_FILENAME "XF86Config.tmp"
-
 /*
  * Define this to have /etc/X11/XF86Config prompted for as the default
  * location to write the XF86Config file to.
@@ -155,11 +147,6 @@ static int getuid() { return 0; }
 
 #define DUMBCONFIG2 "dumbconfig.2"
 #define DUMBCONFIG3 "dumbconfig.3"
-#ifndef __UNIXOS2__
-#define XSERVERNAME_FOR_PROBE "X"
-#else
-#define XSERVERNAME_FOR_PROBE "/usr/X11R6/bin/"__XSERVERNAME__
-#endif
 
 /* some more vars to make path names in texts more flexible. OS/2 users
  * may be more irritated than Unix users
@@ -175,13 +162,32 @@ static int getuid() { return 0; }
 #else
 # define TREEROOTDOC		TREEROOTLX "/doc"
 #endif
+#ifdef XFONTDIR
+# define TREEROOTFONT		XFONTDIR
+#else
+# define TREEROOTFONT		TREEROOTLX "/fonts"
+#endif
 #define MODULEPATH		TREEROOT "/lib/modules"
 
+#ifndef __UNIXOS2__
+#define XSERVERNAME_FOR_PROBE "X"
+#else
+#define XSERVERNAME_FOR_PROBE PROJECTROOT"/bin/"__XSERVERNAME__
+#endif
+
 #ifndef XCONFIGFILE
 #define XCONFIGFILE		"XF86Config"
 #endif
 #define CONFIGNAME		XCONFIGFILE
 
+/*
+ * This is the filename of the temporary XF86Config file that is written
+ * when the program is told to probe clocks (which can only happen for
+ * root).
+ */
+#define TEMPORARY_XF86CONFIG_DIR_PREFIX "/tmp/."XCONFIGFILE
+#define TEMPORARY_XF86CONFIG_FILENAME XCONFIGFILE".tmp"
+
 #ifndef XF86_VERSION_MAJOR
 #ifdef XVERSION
 #if XVERSION > 40000000
@@ -241,7 +247,7 @@ char *temp_dir = "";
 int card_selected;	/* Card selected from database. */
 
 
-static void write_XF86Config(char *filename);
+static int write_XF86Config(char *filename);
 
 
 /*
@@ -635,7 +641,7 @@ mouse_configuration(void) {
 		config_emulate3buttons = 0;
 	printf("\n");
 
-#if (defined(sun) && defined(__i386))
+#if (defined(sun) && (defined(__i386) || defined(__x86)))
 	/* SPARC & USB mice (VUID or AUTO protocols) default to /dev/mouse, 
 	   but PS/2 mice default to /dev/kdmouse */
 	if ((config_mousetype != M_AUTO) && (config_mousetype != M_VUID)) {
@@ -854,8 +860,7 @@ static char *monitorintro_text =
 "which is the rate at which scanlines are displayed.\n"
 "\n"
 "The valid range for horizontal sync and vertical sync should be documented\n"
-"in the manual of your monitor. If in doubt, check the monitor database\n"
-TREEROOTDOC "/Monitors to see if your monitor is there.\n"
+"in the manual of your monitor.\n"
 "\n";
 
 static char *hsyncintro_text =
@@ -1370,7 +1375,7 @@ static int exists_dir(char *name) {
 	return S_ISDIR(sbuf.st_mode) ? 1 : 0;
 }
 
-static void 
+static int 
 screen_configuration(void) {
 	int i, c/*, np*/;
 	char s[80];
@@ -1533,8 +1538,8 @@ screen_configuration(void) {
 		config_virtualy8bpp = 400;
 	}
 	else {
-		printf("Fatal error: Invalid amount of video memory.\n");
-		exit(-1);
+		printf("Invalid amount of video memory. Please try again\n");
+		return(1);
 	}
 
 #if 0
@@ -1821,6 +1826,7 @@ skipclockprobing:
 		if (answerisyes(s))
 			config_virtual = 1;
 	}
+	return(0);
 }
 
 static char *defaultdepthtext = 
@@ -1966,17 +1972,17 @@ static char *XF86Config_firstchunk_text =
 
 static char *XF86Config_fontpaths[] = 
 {
-/*	"    FontPath	\"" TREEROOTLX "/fonts/75dpi/\"\n"*/
-    "/fonts/local/",
-	"/fonts/misc/",
-	"/fonts/75dpi/:unscaled",
-	"/fonts/100dpi/:unscaled",
-	"/fonts/Speedo/",
-	"/fonts/Type1/",
-	"/fonts/TrueType/",
-	"/fonts/freefont/",
-	"/fonts/75dpi/",
-	"/fonts/100dpi/",
+/*	"    FontPath	\"" TREEROOTFONT "/75dpi/\"\n"*/
+    "/local/",
+	"/misc/",
+	"/75dpi/:unscaled",
+	"/100dpi/:unscaled",
+	"/Speedo/",
+	"/Type1/",
+	"/TrueType/",
+	"/freefont/",
+	"/75dpi/",
+	"/100dpi/",
 	0 /* end of fontpaths */
 };
 
@@ -2001,18 +2007,18 @@ static char *XF86Config_fontpathchunk_text =
 "\n"
 "#    Option \"NoTrapSignals\"\n"
 "\n"
-"# Uncomment this to disable the <Crtl><Alt><Fn> VT switch sequence\n"
+"# Uncomment this to disable the <Ctrl><Alt><Fn> VT switch sequence\n"
 "# (where n is 1 through 12).  This allows clients to receive these key\n"
 "# events.\n"
 "\n"
 "#    Option \"DontVTSwitch\"\n"
 "\n"
-"# Uncomment this to disable the <Crtl><Alt><BS> server abort sequence\n"
+"# Uncomment this to disable the <Ctrl><Alt><BS> server abort sequence\n"
 "# This allows clients to receive this key event.\n"
 "\n"
 "#    Option \"DontZap\"\n"
 "\n"
-"# Uncomment this to disable the <Crtl><Alt><KP_+>/<KP_-> mode switching\n"
+"# Uncomment this to disable the <Ctrl><Alt><KP_+>/<KP_-> mode switching\n"
 "# sequences.  This allows clients to receive these key events.\n"
 "\n"
 "#    Option \"Dont Zoom\"\n"
@@ -2505,7 +2511,7 @@ write_fontpath_section(FILE *f)
 #endif
 
 	for (i=0; XF86Config_fontpaths[i]; i++) {
-		strcpy(cur,TREEROOTLX);
+		strcpy(cur,TREEROOTFONT);
 		strcat(cur,XF86Config_fontpaths[i]);
 		/* remove a ':' */
 		colon = strchr(cur+2,':'); /* OS/2: C:/...:scaled */
@@ -2518,12 +2524,12 @@ write_fontpath_section(FILE *f)
 		hash = exists_dir(cur) ? "" : "#";
 		fprintf(f,"%s    FontPath   \"%s%s\"\n",
 			hash,
-			TREEROOTLX,
+			TREEROOTFONT,
 			XF86Config_fontpaths[i]);
 	}
 }
 
-static void 
+static int 
 write_XF86Config(char *filename)
 {
 	FILE *f;
@@ -2539,7 +2545,7 @@ write_XF86Config(char *filename)
 		if (getuid() != 0)
 			printf("Maybe you need to be root to write to the specified directory?\n");
 #endif
-		exit(-1);
+		return(1);
 	}
 
 	fprintf(f, "%s", XF86Config_firstchunk_text);
@@ -2732,6 +2738,7 @@ write_XF86Config(char *filename)
 	fprintf(f, serverlayout_section_text2);
 
         fclose(f);
+	return(0);
 }
 
 static char *
@@ -2952,7 +2959,7 @@ main(int argc, char *argv[]) {
 
 	emptylines();
 
- 	screen_configuration();
+ 	while(screen_configuration()){};
 
 	emptylines();
 
@@ -2960,7 +2967,7 @@ main(int argc, char *argv[]) {
 
 	emptylines();
 
-	write_XF86Config(ask_XF86Config_location());
+	while(write_XF86Config(ask_XF86Config_location())){};
 
 	printf("%s", finalcomment_text);
 
